@@ -35,8 +35,28 @@ survspat <- function(   formula,
                         control=inference.control(gridded=FALSE),
                         boundingbox=NULL){
 
+    if(!is.null(control$nis) & !control$gridded){
+        stop("If control$nis is not null, then you must use gridded inference, please set control$gridded to TRUE and select an appropriate cell width.")
+    }
+
+    imputeMode <- FALSE
+
+    if(!is.null(control$nis)){
+        if(!is.list(control$nis)){
+            stop("control$nis must be a list of matrices.")
+        }
+        if(is.null(control$olinfo)){
+            stop("Require overlay information for control$nis, generated by function prepDataSpatial")
+        }
+
+        imputeMode <- TRUE
+    }
+
     if(!is.null(boundingbox)){
-        boundingbox <- spTransform(boundingbox,CRS(proj4string(data)))
+        try(boundingbox <- spTransform(boundingbox,CRS(proj4string(data))))
+        if(inherits(boundingbox,"try-error")){
+            boundingbox <- spTransform(boundingbox,CRS(proj4string(shape)))
+        }
     }
 
     formula <- as.formula(formula)
@@ -60,6 +80,14 @@ survspat <- function(   formula,
         latentmode <- "SPDE"
     }
 
+    if(imputeMode){
+        latentmode <- "points"
+
+        coordinates(data) <- dat2coords(control$nis,control$olinfo,data[[ids$dataid]])
+        proj4string(data) <- CRS(proj4string(shape))
+        #browser()
+    }
+
     if(inherits(data,"SpatialPointsDataFrame")&!is.null(shape)){
         if(latentmode!="SPDE"){
             warning("Non NULL shape with spatial points survival data, are you sure you want to proceed ???",immediate.=TRUE)
@@ -521,6 +549,70 @@ survspat <- function(   formula,
 
     while(nextStep(mcmcloop)){
 
+        if(imputeMode){
+            if(iteration(mcmcloop)%%101==0){
+                data <- as.data.frame(data)
+                coordinates(data) <- dat2coords(control$nis,control$olinfo,data[[ids$dataid]])
+                proj4string(data) <- CRS(proj4string(shape))
+                control$idx <- over(data,geometry(spix))
+                control$uqidx <- unique(control$idx)
+
+                if(control$censoringtype=="left" | control$censoringtype=="right"){
+                    control$idxi <- list()
+                    control$idxicensored <- list()
+                    control$idxinotcensored <- list()
+                    lapply(control$uqidx,function(i){control$idxi[[i]] <<- which(control$idx==i)})
+                    lapply(control$uqidx,function(i){control$idxicensored[[i]] <<- control$censored & control$idx==i})
+                    lapply(control$uqidx,function(i){control$idxinotcensored[[i]] <<- control$notcensored & control$idx==i})
+
+                    if(control$Ctest){
+                        control$idxicensored <- lapply(control$idxicensored,function(x){try(which(x),silent=TRUE)})
+                    }
+                    if(control$Utest){
+                        control$sumidxinotcensored <- lapply(control$idxinotcensored,function(x){try(sum(x),silent=TRUE)})
+                        control$idxinotcensored <- lapply(control$idxinotcensored,function(x){try(which(x),silent=TRUE)})
+                    }
+                }
+                else{
+                    control$idxirightcensored <- list()
+                    control$idxinotcensored <- list()
+                    control$idxileftcensored <- list()
+                    control$idxiintervalcensored <- list()
+
+                    lapply(control$uqidx,function(i){control$idxirightcensored[[i]] <<- control$rightcensored & control$idx==i})
+                    lapply(control$uqidx,function(i){control$idxinotcensored[[i]] <<- control$notcensored & control$idx==i})
+                    lapply(control$uqidx,function(i){control$idxileftcensored[[i]] <<- control$leftcensored & control$idx==i})
+                    lapply(control$uqidx,function(i){control$idxiintervalcensored[[i]] <<- control$intervalcensored & control$idx==i})
+
+                    if(control$Rtest){
+                        control$idxirightcensored <- lapply(control$idxirightcensored,function(x){try(which(x),silent=TRUE)})
+                    }
+                    if(control$Utest){
+                        control$idxinotcensored <- lapply(control$idxinotcensored,function(x){try(which(x),silent=TRUE)})
+                    }
+                    if(control$Ltest){
+                        control$idxileftcensored <- lapply(control$idxileftcensored,function(x){try(which(x),silent=TRUE)})
+                    }
+                    if(control$Itest){
+                        control$idxiintervalcensored <- lapply(control$idxiintervalcensored,function(x){try(which(x),silent=TRUE)})
+                    }
+                }
+
+                oldlogpost <- LOGPOST(  surv=survivaldata, # re-compute likelihood with new data
+                                        X=X,
+                                        beta=beta,
+                                        omega=omega,
+                                        eta=eta,
+                                        gamma=gamma,
+                                        priors=priors,
+                                        cov.model=cov.model,
+                                        u=u,
+                                        control=control,
+                                        gradient=TRUE)
+
+            }
+        }
+
         stuffpars <- c(beta,omega,eta)
         propmeanpars <- stuffpars + (h/2)*SIGMApars%*%oldlogpost$grad[1:(lenbeta+lenomega+leneta)]
         newstuffpars <- propmeanpars + sqrt(h)*cholSIGMApars%*%rnorm(lenbeta+lenomega+leneta)
